
body {
	margin:0px;
	padding:0px;
	font-family:verdana, arial, helvetica, sans-serif; 
	color:#333;
	background-color:white;
	}
pre {
  background: #eeeeee;
  border: 1px solid #888888;
  color: black;
  padding: 1em;
  white-space: pre;
}
h1 {
	margin:5px 0px 5px 0px;
	padding:0px;
	font-size:20px;
	line-height:28px;
	font-weight:900;
	color:#44f;
	}
h2 {
	margin:5px 0px 5px 0px;
	padding:0px;
	font-size:17px;
	line-height:28px;
	font-weight:900;
	color:#226;
	}
h3 {
	margin:5px 0px 5px 0px;
	padding:0px;
	font-size:15px;
	line-height:28px;
	font-weight:900;
	}
p
{
	margin:0px 0px 16px 0px;
	font:11px/20px verdana, arial, helvetica, sans-serif;
	padding:0px;
}
table 
{
    font-size: 10pt; 
	color: #000000;
}
td{border:1px solid #999;}

table.pymenu {color: #000000; background-color: #99ccff}
th.pymenu {color: #ffffff; background-color: #003366}

.code 
{
	font-family: "Lucida Console", monospace; font-weight: bold;
	color: #007700; background-color: #eeeeee
}

#Content>p {margin:0px;}
#Content>p+p {text-indent:30px;}

a {
	text-decoration:none;
	font-weight:600;
	font-family:verdana, arial, helvetica, sans-serif;
	color: #900;
}
//a:link {color:#09c;}
//a x:visited {color:#07a;}
a:hover {background-color:#ee0;}

#Header {
	margin:10px 0px 10px 0px;
	padding:10px 0px 10px 20px;
	/* For IE5/Win's benefit height = [correct height] + [top padding] + [top and bottom border widths] */
	height:33px; /* 14px + 17px + 2px = 33px */
	border-style:solid;
	border-color:black;
	border-width:1px 0px; /* top and bottom borders: 1px; left and right borders: 0px */
	line-height:33px;
	background-color:#eee;
	height:66px; /* the correct height */
	}

#Content {
	margin:0px 210px 50px 10px;
	padding:10px;
	}

#Menu {
	position:absolute;
	top:100px;
	right:20px;
	width:172px;
	padding:10px;
	background-color:#eee;
	border:1px solid #999; // dashed #999;
	line-height:17px;
	width:150px;
	font-size:11px;
	}
